// Forlix DeadChat
// http://forlix.org/, df@forlix.org
//
// Copyright (c) 2008-2011 Dominik Friedrichs

#include <sourcemod>
#include <cstrike>

#pragma semicolon 1

#define PLUGIN_VERSION            "1.16"
#define PLUGIN_VERSION_CVAR       "forlix_deadchat_version"

#define MAX_MTYPE_LEN             32
#define MAX_NAME_LEN              32
#define MAX_MSG_LEN               128

#define BCTYPE_ALL                1
#define BCTYPE_TEAM               2

// convar defaults
#define RELAY_MODE                "1"

public Plugin:myinfo =
{
  name = "Forlix DeadChat",
  author = "Forlix (Dominik Friedrichs)",
  description = "Relays chat messages of dead players and spectators to alive players",
  version = PLUGIN_VERSION,
  url = "http://forlix.org/"
};

new Handle:h_relay_mode = INVALID_HANDLE;
new relay_mode = 0;

new p_bctype[MAXPLAYERS+1];


public OnPluginStart()
{
  RegConsoleCmd("say", Command_Say);
  RegConsoleCmd("say_team", Command_SayTeam);

  HookEvent("player_say", Event_PlayerSay);

  new Handle:version_cvar = CreateConVar(PLUGIN_VERSION_CVAR,
  PLUGIN_VERSION,
  "Forlix DeadChat plugin version",
  FCVAR_PLUGIN|FCVAR_SPONLY|FCVAR_NOTIFY|FCVAR_PRINTABLEONLY);

  SetConVarString(version_cvar, PLUGIN_VERSION, false, false);

  h_relay_mode = CreateConVar("forlix_deadchat_relay_mode",
  RELAY_MODE,
  "Relay all messages to alive players, respecting team-only flag (1), Disable relaying (0)",
  0, true, 0.0, true, 1.0);

  HookConVarChange(h_relay_mode, MyConVarChanged);

  // manually trigger convar readout
  MyConVarChanged(INVALID_HANDLE, "0", "0");

  return;
}


public OnClientConnected(client)
{
  p_bctype[client] = 0;
  return;
}


public Action:Command_Say(client,
                          args)
{
  p_bctype[client] = BCTYPE_ALL;
  return(Plugin_Continue);
}


public Action:Command_SayTeam(client,
                              args)
{
  p_bctype[client] = BCTYPE_TEAM;
  return(Plugin_Continue);
}


public Event_PlayerSay(Handle:event,
                       const String:Event_mtype[],
                       bool:dontBroadcast)
{
  new client = GetClientOfUserId(GetEventInt(event, "userid"));

  if(!relay_mode
  || !client
  || !p_bctype[client]
  || dontBroadcast
  || IsPlayerAlive(client))
    return;

  decl String:mtype[MAX_MTYPE_LEN];
  decl String:name[MAX_NAME_LEN];
  decl String:text[MAX_MSG_LEN];

  decl tx_clients[MAXPLAYERS];
  new tx_count = 0;

  if(!GetClientName(client, name, sizeof(name)))
    return;

  GetEventString(event, "text", text, sizeof(text));

  new team = GetClientTeam(client);

  switch(p_bctype[client])
  {
    case BCTYPE_ALL:
    // broadcast message
    {
      if(team == CS_TEAM_SPECTATOR)
        mtype = "Cstrike_Chat_AllSpec";
      else
        mtype = "Cstrike_Chat_AllDead";

      // build list of clients to relay to
      for(new i = 1; i <= MaxClients; i++)
      if(IsClientInGame(i)
      && IsPlayerAlive(i))
        tx_clients[tx_count++] = i;
    }

    case BCTYPE_TEAM:
    // team-only message
    {
      switch(team)
      // only process teams that can have alive players
      // eg. spectators are all dead, no need to relay
      {
        case CS_TEAM_T:
          mtype = "Cstrike_Chat_T_Dead";

        case CS_TEAM_CT:
          mtype = "Cstrike_Chat_CT_Dead";

        default:
          return;
      }

      // build list of clients to relay to
      for(new i = 1; i <= MaxClients; i++)
      if(IsClientInGame(i)
      && IsPlayerAlive(i)
      && GetClientTeam(i) == team)
        tx_clients[tx_count++] = i;
    }

    default:
      return;
  }

  if(!tx_count)
  // no clients to relay to
    return;

  // finally, relay the message to clients in list
  new Handle:h_saytext2 = StartMessage("SayText2",
  tx_clients,
  tx_count,
  USERMSG_RELIABLE|USERMSG_BLOCKHOOKS);

  if(h_saytext2 != INVALID_HANDLE)
  {
    BfWriteByte(h_saytext2, client);
    BfWriteByte(h_saytext2, true);
    BfWriteString(h_saytext2, mtype);
    BfWriteString(h_saytext2, name);
    BfWriteString(h_saytext2, text);
    EndMessage();
  }

  p_bctype[client] = 0;
  return;
}

public MyConVarChanged(Handle:convar,
                       const String:oldValue[],
                       const String:newValue[])
{
  relay_mode = GetConVarInt(h_relay_mode);
  return;
}
